@@ -5,12 +5,16 @@ from threading import Thread
 import time
 import holoviews as hv
 import panel as pn
+
 try:
     from cartopy import crs
     import cartopy.crs as ccrs
+
     projection = ccrs.PlateCarree()
 except:
-    warnings.warn("Unable to import cartopy. For better plots install cartopy or check cartopy installation!")
+    warnings.warn(
+        "Unable to import cartopy. For better plots install cartopy or check cartopy installation!"
+    )
     projection = None
 
 import pandas as pd
@@ -27,6 +31,17 @@ from ncplot.utils import get_dims, check_lon, check_lat
 hv.extension("bokeh")
 hv.Store.renderers
 
+def as_raster():
+    """
+    Simple hack to rasterize when datashader is available.
+    This is to get around Python 3.11 datashader nonavailability
+    """
+    try:
+        import datashader
+        return True
+    except:
+        return False
+
 
 def get_coastline(ds, lon_name, lat_name):
     """
@@ -131,7 +146,7 @@ def ctrc():
     print("Press Ctrl+C to stop plotting server")
 
 
-def in_notebook(out = None):
+def in_notebook(out=None):
     """
     Returns ``True`` if the module is running in IPython kernel,
     ``False`` if in IPython shell or other Python shell.
@@ -146,7 +161,7 @@ def in_notebook(out = None):
     return "ipykernel" in sys.modules
 
 
-def view(x, vars=None, autoscale=True,out = None, **kwargs):
+def view(x, vars=None, autoscale=True, out=None, **kwargs):
     """
     Plot the contents of a NetCDF out
     Parameters
@@ -164,11 +179,18 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
     if "clim" in kwargs:
         autoscale = False
 
+    coastline = False
+
     if out is not None:
         if out.endswith(".html") is False:
             raise ValueError("out name must end with html")
 
-    coastline = True
+    for kk in kwargs.keys():
+        if kk == "coast":
+            coastline = kwargs[kk]
+
+    if "coast" in kwargs.keys():
+        kwargs.pop("coast")
     if type(x) is xr.core.dataarray.DataArray:
         x = x.to_dataset()
 
@@ -182,6 +204,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
         try:
             try:
                 import nctoolkit as nc
+
                 ds = nc.open_data(x)
                 nc_vars = ds.variables
                 ds = ds.to_xarray()
@@ -273,7 +296,11 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
 
     switch_coords = False
 
-    rasterize = True
+    try:
+        #import datashader
+        rasterize = True
+    except:
+        rasterize = False
 
     if quadmesh:
         switch_coords = True
@@ -457,7 +484,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
         df = ds.to_dataframe()
         if nc_vars is not None:
             cols = [x for x in list(df.columns) if x in nc_vars]
-            df = df.loc[:,cols]
+            df = df.loc[:, cols]
 
         df = df.reset_index().drop_duplicates().reset_index()
 
@@ -476,7 +503,8 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
         intplot = df.hvplot(
             groupby="variable",
             dynamic=True,
-            responsive=(in_notebook() is False), **kwargs,
+            responsive=(in_notebook() is False),
+            **kwargs,
         )
         if in_notebook(out):
             if out is None:
@@ -540,9 +568,13 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         quadmesh = True
 
                 if quadmesh:
-                    intplot = ds.hvplot.quadmesh(x_var, y_var, vars, cmap="viridis", **kwargs)
+                    intplot = ds.hvplot.quadmesh(
+                        x_var, y_var, vars, cmap="viridis", **kwargs
+                    )
                 else:
-                    intplot = ds.hvplot.image(x_var, y_var, vars, cmap="viridis", **kwargs)
+                    intplot = ds.hvplot.image(
+                        x_var, y_var, vars, cmap="viridis", **kwargs
+                    )
 
             else:
 
@@ -558,24 +590,26 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 # figure out if it needs to be quadmesh
                 if x_var == time_name:
                     quadmesh = True
-                x_vals = ds[x_var].values
+                if not quadmesh:
+                    x_vals = ds[x_var].values
 
-                if len(x_vals) > 2:
-                    if (
-                        np.nanmax(x_vals[0:-2] - x_vals[1:-1]).astype("float")
-                        - np.nanmin(x_vals[0:-2] - x_vals[1:-1]).astype("float")
-                        > 0
-                    ):
-                        quadmesh = True
-                y_vals = ds[y_var].values
+                    if len(x_vals) > 2:
+                        if (
+                            np.nanmax(x_vals[0:-2] - x_vals[1:-1]).astype("float")
+                            - np.nanmin(x_vals[0:-2] - x_vals[1:-1]).astype("float")
+                            > 0
+                        ):
+                            quadmesh = True
+                if not quadmesh:
+                    y_vals = ds[y_var].values
 
-                if len(y_vals) > 2:
-                    if (
-                        np.nanmax(y_vals[0:-2] - y_vals[1:-1]).astype("float")
-                        - np.nanmin(y_vals[0:-2] - y_vals[1:-1]).astype("float")
-                        > 0
-                    ):
-                        quadmesh = True
+                    if len(y_vals) > 2:
+                        if (
+                            np.nanmax(y_vals[0:-2] - y_vals[1:-1]).astype("float")
+                            - np.nanmin(y_vals[0:-2] - y_vals[1:-1]).astype("float")
+                            > 0
+                        ):
+                            quadmesh = True
 
                 if self_max > 0 and self_min < 0:
                     if quadmesh:
@@ -584,7 +618,8 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                             y_var,
                             vars,
                             cmap="RdBu_r",
-                            responsive=(in_notebook() is False), **kwargs,
+                            responsive=(in_notebook() is False),
+                            **kwargs,
                         ).redim.range(**{vars: (-v_max, v_max)})
                     else:
                         intplot = ds.hvplot.image(
@@ -593,14 +628,19 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                             vars,
                             cmap="RdBu_r",
                             rasterize=False,
-                            responsive=(in_notebook() is False), **kwargs,
+                            responsive=(in_notebook() is False),
+                            **kwargs,
                         ).redim.range(**{vars: (-v_max, v_max)})
 
                 else:
                     if quadmesh:
-                        intplot = ds.hvplot.quadmesh(x_var, y_var, vars, cmap="viridis", **kwargs)
+                        intplot = ds.hvplot.quadmesh(
+                            x_var, y_var, vars, cmap="viridis", **kwargs
+                        )
                     else:
-                        intplot = ds.hvplot.image(x_var, y_var, vars, cmap="viridis", **kwargs)
+                        intplot = ds.hvplot.image(
+                            x_var, y_var, vars, cmap="viridis", **kwargs
+                        )
 
             if in_notebook(out):
                 if out is None:
@@ -647,7 +687,6 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 time_in = True
                 possible += 1
 
-
         if time_name in coord_list and time_in and non_map:
 
             if coord_df.query("coord == @time_name").length.values > 1:
@@ -710,11 +749,16 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
 
                     if quadmesh:
                         intplot = ds.hvplot.quadmesh(
-                            x_var, y_var, vars, cmap="viridis", rasterize=True, **kwargs
+                            x_var, y_var, vars, cmap="viridis", rasterize=rasterize, **kwargs
                         )
                     else:
                         intplot = ds.hvplot.image(
-                            x_var, y_var, vars, cmap="viridis", rasterize=False, **kwargs
+                            x_var,
+                            y_var,
+                            vars,
+                            cmap="viridis",
+                            rasterize=False,
+                            **kwargs,
                         )
 
                     if in_notebook(out):
@@ -763,7 +807,8 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
             .hvplot(
                 groupby="variable",
                 dynamic=True,
-                responsive=(in_notebook() is False), **kwargs,
+                responsive=(in_notebook() is False),
+                **kwargs,
             )
         )
         if in_notebook(out):
@@ -792,7 +837,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 except:
                     projection = None
                     coastline = False
-                    
+
             else:
                 projection = None
 
@@ -806,7 +851,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 projection=projection,
                 rasterize=rasterize,
                 responsive=in_notebook() is False,
-                **kwargs
+                **kwargs,
             )
         else:
 
@@ -830,7 +875,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 rasterize=False,
                 projection=projection,
                 responsive=in_notebook() is False,
-                **kwargs
+                **kwargs,
             )
 
         if in_notebook(out):
@@ -884,7 +929,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         projection=projection,
                         rasterize=rasterize,
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     ).redim.range(**{vars: (-v_max, v_max)})
                 else:
                     intplot = ds.hvplot.quadmesh(
@@ -897,7 +942,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         projection=projection,
                         rasterize=rasterize,
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     )
             else:
                 if coastline:
@@ -921,7 +966,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         rasterize=False,
                         cmap="RdBu_r",
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     ).redim.range(**{vars: (-v_max, v_max)})
                 else:
                     intplot = ds.hvplot.image(
@@ -934,7 +979,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         rasterize=False,
                         cmap="RdBu_r",
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     )
 
             if in_notebook(out):
@@ -963,7 +1008,6 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 else:
                     projection = None
 
-
                 if autoscale:
                     intplot = ds.hvplot.quadmesh(
                         lon_name,
@@ -975,7 +1019,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         projection=projection,
                         rasterize=rasterize,
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     ).redim.range(**{vars: (self_min.values, v_max)})
                 else:
                     intplot = ds.hvplot.quadmesh(
@@ -988,7 +1032,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         projection=projection,
                         rasterize=rasterize,
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     )
 
             else:
@@ -1014,7 +1058,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         rasterize=False,
                         projection=projection,
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     ).redim.range(**{vars: (self_min.values, v_max)})
                 else:
                     intplot = ds.hvplot.image(
@@ -1027,7 +1071,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         rasterize=False,
                         projection=projection,
                         responsive=(in_notebook() is False),
-                        **kwargs
+                        **kwargs,
                     )
 
             if in_notebook(out):
